@@ -16,13 +16,19 @@
 #include <linux/kdev_t.h>
 #undef __KERNEL__
 
-
 #define DEVDIR		"/dev"
-#define DEVTAB		"/etc/psdevtab"
+#define DEVTAB		"psdevtab"
+static char *devtab_paths[] = {
+    "/etc/" DEVTAB,
+    "%s/."  DEVTAB,
+    NULL
+};
+#define DEVINITIALMODE	0664
 #define DEV_MAX_PATH	(5+256)
 #define DEV_NAME_MAX	8
 
-static dev_t major[] = { 2, 3, 4, 5, 19, 20 };
+static dev_t major[] = { 2, 3, 4, 5, 19, 20, 22, 23, 24, 25, 32, 33,
+			46, 47, 48, 49 };
 #define Nminor		256
 #define Nmajor		(sizeof(major)/sizeof(dev_t))
 #define Ndev		(Nmajor*Nminor)
@@ -43,13 +49,8 @@ static int   init_devtab (void);
    many-to-one: -1 on failed match.
 */
 dev_t name_to_dev(char* name) {
-    struct stat sbuf;
-    if (*name) { /* use named tty */
-	if (stat(name_to_path(name), &sbuf) < 0) return -1;
-    }
-    else if (fstat(0, &sbuf) < 0) return -1; /* use current tty */
-    if (!S_ISCHR(sbuf.st_mode)) return -1;
-    return sbuf.st_rdev;
+    static struct stat sbuf;
+    return (stat(name_to_path(name), &sbuf) < 0) ? -1 : sbuf.st_rdev;
 }
 
 /* find m in a[] assuming a is sorted into ascending order */
@@ -63,8 +64,6 @@ static __inline__ int lookup(dev_t m, dev_t* a, int n) {
 
 /* Device Number -> Name Map
    one-to-many: first directory order match in DEVDIR, "" on failed match.
-      Accuracy is guaranteed through expensive stat's only when DEVTAB is
-      older than DEVDIR when this function is first called.
 */
 char* dev_to_name(dev_t num) {
     static char	rval[DEV_NAME_MAX+1];
@@ -78,79 +77,79 @@ char* dev_to_name(dev_t num) {
     return rval;
 }
 
-static int dev_to_devtab(void);
+static int dev_to_devtab(int);
 
 static int init_devtab(void) {
-    static struct stat sbuf, sbuf2;
+    static struct stat sbuf, lbuf;
     static int fd;
-    stat(DEVDIR, &sbuf2);			/* stat DEVDIR */
-    if ( (fd = open(DEVTAB, O_RDONLY)) < 0	/* open DEVTAB file */
-	 || fstat(fd, &sbuf) < 0		/* fstat it */
-	 || sbuf.st_size != Ndevtab		/* make sure it's the right size */
-	 || sbuf.st_mtime < sbuf2.st_mtime	/* and more recent than DEVDIR */
-	 || (devtab = mmap(0, Ndevtab, PROT_READ, MAP_FILE|MAP_SHARED, fd, 0))==(caddr_t) -1)
-    {	/* one of the above failed, so don't mmap() the file */
-	close(fd);		 	/* build DEVTAB from DEVDIR */
-	if (dev_to_devtab()) {
-	    devtab_initialized = 1;			 
-	    return 1;
-	} else
-	    return 0;
+    char **fmt, path[64], *HOME = getenv("HOME") ? getenv("HOME") : "";
+    for (fmt = devtab_paths; *fmt; fmt++) {
+	snprintf(path, sizeof path, *fmt, HOME);
+	lbuf.st_ino = 0; /* initialize for test later */
+	if (lstat(path, &lbuf) >= 0 && S_ISLNK(lbuf.st_mode))
+	    /* don't follow symlinks */
+	    continue;
+	if ( (fd = open(path, O_RDONLY)) < 0	/* open DEVTAB file */
+	     || fstat(fd, &sbuf) < 0		/* fstat it */
+	     || (lbuf.st_ino && (sbuf.st_ino != lbuf.st_ino)) /* race */
+	     || sbuf.st_nlink > 1		/* hardlink attack */
+	     || sbuf.st_size != Ndevtab		/* make sure it's the right size */
+	     || (devtab = mmap(0, Ndevtab, PROT_READ, MAP_SHARED, fd, 0)) == (caddr_t) -1
+	     || close(fd) == -1)
+	    {	/* could not open for read, attempt to fix/create */
+		int oumsk = umask(0);
+		if (devtab)
+		    munmap(devtab, Ndevtab);
+		if (((fd = open(path, O_RDWR|O_TRUNC|O_CREAT, DEVINITIALMODE)) == -1 &&
+		     (unlink(path), fd = open(path, O_RDWR|O_TRUNC|O_CREAT, DEVINITIALMODE)) == -1)
+		     || !dev_to_devtab(fd)) {
+		    close(fd);          /* either both opens failed or the constructor failed */
+		    unlink(path);	/* in case we created but could not fill a file */
+		    umask(oumsk);
+		    continue;
+		} else {
+		    devtab_initialized = 1;
+		    close(fd);
+		    umask(oumsk);
+		    return 1;
+		}
+	    }
+	else
+	    return devtab_initialized = 1;
     }
-    devtab_initialized = 1;
-    return 1;
+    return devtab_initialized;
 }
 
 /* stat every file in DEVDIR saving its basename in devtab[] if it has
    a MAJOR(st_rdev) in our list of majors. return 0 on error otherwise 1. */
-
-/* cannot update DEVTAB, malloc devtab instead */
-#define ABORT_FILE_UPDATE	\
-    close(fd);			\
-    got_fd = 0;			\
-    devtab = malloc(Ndevtab);	\
-    goto devtab_allocated;
-
-static int dev_to_devtab(void) {
+static int dev_to_devtab(int fd) {
     static struct stat sbuf;
-    int fd, got_fd = 1, i, old_umask = umask(0);
+    int i;
     dev_t m;
     struct dirent* ent;
     DIR* dev;
     
     if (!(dev = opendir(DEVDIR))) {
-	fprintf(stderr, "%s: %s\nCannot continue deducing device number -> name mapping.\n",
+	fprintf(stderr, "%s: %s\nCannot generate device number -> name mapping.\n",
 		DEVDIR, sys_errlist[errno]);
-	umask(old_umask);
 	return 0;
     }
-    /* Try to get an fd open for write on DEVTAB in one of two possible ways */
-    if ((fd = open(DEVTAB, O_RDWR|O_TRUNC)) < 0) {	/* try opening it first */ 
-	unlink(DEVTAB);					/* if that fails unlink+open */
-	if ((fd = open(DEVTAB, O_CREAT|O_RDWR|O_TRUNC, 0)) < 0)
-	    { ABORT_FILE_UPDATE }
+    if (!(devtab = malloc(Ndevtab))) {
+	fprintf(stderr, "%s: could not allocate memory\n", sys_errlist[errno]);
+	return 0;
     }
-    umask(old_umask);					/* not creating any more files */
-    /* Set file to appropriate size, then mmap it. */
-    if (lseek(fd, Ndevtab - 1, SEEK_SET) < 0)
-	{ ABORT_FILE_UPDATE }
-    if (write(fd, "", 1) < 0)
-	{ ABORT_FILE_UPDATE }
-    if ((devtab = mmap(0, Ndevtab, PROT_READ|PROT_WRITE, MAP_FILE|MAP_SHARED, fd, 0))==(caddr_t)-1)
-	{ ABORT_FILE_UPDATE }
-devtab_allocated:
-    memset((void*)devtab, 0, Ndevtab);	/* initialize devtab: necessary? */
+    memset((void*)devtab, 0, Ndevtab);
     while ((ent = readdir(dev))) {	/* loop over all dirents in DEVDIR */
-	if (lstat(name_to_path(ent->d_name), &sbuf) < 0) /* lstat entry (avoid symlinks) */
-	    continue;			    /* failures do not interest us */
-	if (!S_ISCHR(sbuf.st_mode))	    /* only support char special devs */
+	if (lstat(name_to_path(ent->d_name), &sbuf) < 0
+	    || !S_ISCHR(sbuf.st_mode))	    /* only look for char special devs */
 	    continue;			    /* due to overloading of majors */
 	m = MAJOR(sbuf.st_rdev);	    /* copy name to appropriate spot */
 	if ((i = lookup(m, major, Nmajor)) != -1)
 	    strncpy(TAB(i,MINOR(sbuf.st_rdev)), ent->d_name, DEV_NAME_MAX);
     }
     closedir(dev);
-    if (got_fd) close(fd);
+    if (write(fd, devtab, Ndevtab) != Ndevtab) /* probably no disk space */
+	return 0;
     return 1;
 }
 
@@ -188,19 +187,24 @@ int main(int argc, char** argv) {	/* usage: cmd [<major> <minor>|<name>] */
 /*
 Using this program on over 700 files in /dev to perform number->name resolution
 took well under 300 microsecs per device number pair on a Pentium 90.  It is
-somewhat tough to time because once the 3 pages have been marked as mapped in,
-the time is almost zero.  For things like top, this method may even be faster in
-the long run.  Those interested can gprof it for me.  This system has the virtue
-of being nearly perfectly adaptable to individual systems, self updating when
-/dev changes and pretty darn fast when it hasn't.  It will be slow for users
-without perms to change the psdevtab file, though.  An optional ~/.psdevtab may
-be in order.  But really, how often does /dev change on typical systems?
+somewhat tough to time because once the 3 pages have been mapped in, the time is
+almost zero.  For things like top, this method may even be faster in the long
+run.  Those interested can gprof it for me.  This system has the virtue of being
+nearly perfectly adaptable to individual systems, self updating when /dev
+changes and pretty darn fast when it hasn't.  It will be slow for users without
+perms to change the psdevtab file, though.  So this is what I decided was
+reasonable.  If the process does not have perms to create or update
+/etc/psdevtab and it is out of date, we try /tmp/psdevtab.  If /tmp/psdevtab is
+either out of date or unreadable (malicious user creates it and chmods it),
+$HOME/.psdevtab is used.  This secondarily allows for per-user naming of ttys,
+but is really so that at most one user sees only a single delay per /dev
+modification.
 
 To do the timings I did something like this with zsh:
     a=(`ls -l *(%^@/) | awk '{print $5 $6}' | sed 's/,/ /'`);
     time ./test $a
 
-Finally, for lack of a better file for these to be in, we have collected the
+Finally, for lack of a better file for these to be in, I have collected the
 old algorithmic device number <-> device name mappings.
   Let m = major device number and n = minor device number satisfy:
     devno = m<<8 + n , m = devno>>8 , n = devno && 0x00FF, and let
@@ -224,8 +228,7 @@ old algorithmic device number <-> device name mappings.
 	    n=64..128:	cua  + {'0' + (n-64)}
   CYCLADES MULTIPORT:
 	m=19, n:	ttyC + itoa_hex(n)
-	m=20, n: cub + itoa_hex(n)
-*/
+	m=20, n: cub + itoa_hex(n) */
 
 /* Re-implementation of old interface with the new generic functions. */
 
@@ -264,12 +267,3 @@ void dev_to_tty(char *tty, int dev) {
     char* new = abbrev_of_tty(dev_to_name(dev));
     strncpy(tty, new, 4);
 }
-
-/* dev3 was really just the same as this except for the in-place q-mark. */
-char *dev3(char *ttyname) {
-    if (name_to_dev(ttyname) == (dev_t) -1) {
-        strncpy(ttyname, " ? ", 3);	/* no ctty ==> question mark */
-	return ttyname;
-    } else				/* cvt to abbrev if a valid device */
-	return abbrev_of_tty(ttyname);
-}
