@@ -186,6 +186,8 @@ int main(int argc, char **argv) {
 	      case 'S': CL_Sum = 1;				break;
 	      case 'p': CL_pg_shift = 0;			break;
 	      case 'o': CL_sort = !CL_sort;			break;
+	      case 'g': /* ignore for compatibility with people's fingers */
+			break;
 	      case 'O':
 		if (parse_sort_opt(p+1) == -1)
 		    usage("short form sort flag parse error\n");
@@ -220,11 +222,12 @@ int main(int argc, char **argv) {
     if (!CL_sort)	/* since the unsorted mode is intended to be speedy */
 	CL_forest = 0;	/* turn off the expensive forest option as well. */
 
-    if (CL_fmt == PS_L)
+    if (CL_fmt == PS_L) {
 	if (open_psdb())
 	    GL_wchan_nout = 1;
 	else
 	    psdbsucc = 1;
+    }
 
     set_cmdspc(width);
 
@@ -357,9 +360,10 @@ void show_procs(unsigned maxcmd, int do_header, int pflags, void* args, int N) {
  * use abbreviated cmd if requested, NULL list, or singleton NULL string
  */
 void show_cmd_env(char* tskcmd, char** cmd, char** env, unsigned maxch) {
-    if (CL_kern_comm)		/* no () when explicit request for tsk cmd */
+    if (CL_kern_comm) {		/* no () when explicit request for tsk cmd */
 	maxch = print_str(stdout, tskcmd, maxch);
-    else if (!cmd || !*cmd || (!cmd[1] && !*cmd)) {
+	maxch = print_str(stdout, " ", maxch);
+    } else if (!cmd || !*cmd || (!cmd[1] && !*cmd)) {
 	/* no /proc//cmdline ==> bounding () */
 	if (maxch) {
 	    fputc('(', stdout);
