@@ -274,8 +274,8 @@ void setup_terminal(void)
     mr = tgetstr("mr", 0);
     me = tgetstr("me", 0);
 
-    if (ioctl(0, TCGETS, &Savetty) == -1) {
-	perror(PROGNAME ": ioctl() failed");
+    if (tcgetattr(0, &Savetty) == -1) {
+	perror(PROGNAME ": tcgetattr() failed");
 	error_end(errno);
     }
     newtty = Savetty;
@@ -283,11 +283,11 @@ void setup_terminal(void)
     newtty.c_lflag &= ~ECHO;
     newtty.c_cc[VMIN] = 1;
     newtty.c_cc[VTIME] = 0;
-    if (ioctl(0, TCSETSF, &newtty) == -1) {
+    if (tcsetattr(0, TCSAFLUSH, &newtty) == -1) {
 	printf("cannot put tty into raw mode\n");
 	error_end(1);
     }
-    ioctl(0, TCGETS, &Rawtty);
+    tcgetattr(0, &Rawtty);
 
 }
 
@@ -390,16 +390,14 @@ int main(int argc, char **argv)
     sigaction(SIGINT, &sact, NULL);
     sigaction(SIGQUIT, &sact, NULL);
     sact.sa_handler = stop;
+    sact.sa_flags = SA_RESTART;
     sigaction(SIGTSTP, &sact, NULL);
     sact.sa_handler = window_size;
-    sact.sa_flags = SA_RESTART;
     sigaction(SIGWINCH, &sact, NULL);
+    sigaction(SIGCONT, &sact, NULL);
 
     /* loop, collecting process info and sleeping */
     while (1) {
-	if (setjmp(redraw_jmp))
-	    clear_screen();
-
 	/* display the tasks */
 	show_procs();
 	/* sleep & wait for keyboard input */
@@ -426,7 +424,7 @@ void error_end(int rno)
 {
     if (psdbsucc)
         close_psdb();
-    ioctl(0, TCSETSF, &Savetty);
+    tcsetattr(0, TCSAFLUSH, &Savetty);
     PUTP(tgoto(cm, 0, Lines - 1));
     fputs("\r\n", stdout);
     exit(rno);
@@ -438,7 +436,7 @@ void end(int signo)
 {
     if (psdbsucc)
 	close_psdb();
-    ioctl(0, TCSETSF, &Savetty);
+    tcsetattr(0, TCSAFLUSH, &Savetty);
     PUTP(tgoto(cm, 0, Lines - 1));
     fputs("\r\n", stdout);
     exit(0);
@@ -450,15 +448,12 @@ void end(int signo)
 void stop(int signo)
 {
     /* Reset terminal. */
-    if (psdbsucc)
-	close_psdb();
-    ioctl(0, TCSETSF, &Savetty);
+    tcsetattr(0, TCSAFLUSH, &Savetty);
     PUTP(tgoto(cm, 0, Lines - 3));
     fflush(stdout);
     raise(SIGSTOP);
     /* Later... */
-    ioctl(0, TCSETSF, &Rawtty);
-    longjmp(redraw_jmp, 1);
+    ioctl(0, TCSAFLUSH, &Rawtty);
 }
 
 /*
@@ -693,9 +688,9 @@ void change_order(void)
 	PUTP(tgoto(cm, 0, 1));
 	printf("Upper case characters move a field to the left, lower case to the right");
 	fflush(stdout);
-	ioctl(0, TCSETSF, &Rawtty);
+	tcsetattr(0, TCSAFLUSH, &Rawtty);
 	read(0, &c, 1);
-	ioctl(0, TCSETSF, &Savetty);
+	tcsetattr(0, TCSAFLUSH, &Savetty);
 	i = toupper(c) - 'A';
 	if ((p = strchr(Fields, i + 'A')) != NULL) {
 	    if (isupper(c))
@@ -741,9 +736,9 @@ void change_fields(void)
 	PUTP(tgoto(cm, 0, 1));
 	printf("Toggle fields with a-x, any other key to return: ");
 	fflush(stdout);
-	ioctl(0, TCSETSF, &Rawtty);
+	tcsetattr(0, TCSAFLUSH, &Rawtty);
 	read(0, &c, 1);
-	ioctl(0, TCSETSF, &Savetty);
+	tcsetattr(0, TCSAFLUSH, &Savetty);
 	i = toupper(c) - 'A';
 	if (i >= 0 && i < sizeof headers / sizeof headers[0]) {
 	    row = i % (Lines - 3) + 3;
@@ -945,11 +940,8 @@ void show_task_info(proc_t *task, int pmem)
 	  case P_COMMAND:
 	    if (!show_cmd && task->cmdline && *(task->cmdline)) {
 	        j=0;
-	        while(((task->cmdline)[j] != NULL) && (strlen(tmp3)<1024)){
-		    if (j > 0)
-			strcat(tmp3, " ");
-		    strcat(tmp3,(task->cmdline)[j]);
-		    strcat(tmp3, " ");
+	        while(((task->cmdline)[j] != NULL) && (strlen(tmp3)<1024)) {
+		    strncat(tmp3, (task->cmdline)[j], 1023);
 		    j++; 
 	        }
 	        cmdptr = tmp3;
@@ -1273,7 +1265,7 @@ void do_key(char c)
      * attributes always be normal?  Does it matter?  I suppose the
      * shell will be set up the way the user wants it.)
      */
-    ioctl(0, TCSETS, &Savetty);
+    tcsetattr(0, TCSANOW, &Savetty);
 
     /*
      * Handle the rest of the commands.
@@ -1298,7 +1290,7 @@ void do_key(char c)
 	PUTP(me);
 	fputs("\n\n", stdout);
 	printf("%s\n\nPress any key to continue", Secure ? SECURE_HELP_SCREEN : HELP_SCREEN);
-	ioctl(0, TCSETS, &Rawtty);
+	tcsetattr(0, TCSANOW, &Rawtty);
 	(void) getchar();
 	break;
       case 'i':
@@ -1488,7 +1480,7 @@ void do_key(char c)
     /*
      * Return to raw mode.
      */
-    ioctl(0, TCSETS, &Rawtty);
+    tcsetattr(0, TCSANOW, &Rawtty);
     return;
 }
 
