@@ -80,6 +80,8 @@ int main(int argc, char *argv[]) {
   set_linux_version();
   setlinebuf(stdout);
   argc=0; /* redefined as number of integer arguments */
+  per=1; 
+  num=0;
   for (argv++;*argv;argv++) {
     if ('-' ==(**argv)) {
       switch (*(++(*argv))) {
@@ -96,26 +98,21 @@ int main(int argc, char *argv[]) {
       }
     }
     else {
-      if (!sscanf(*argv,"%lu",args+argc++)) usage();
+      argc++;
+      switch (argc) {
+      case 1:
+        if ((per = atoi(*argv)) == 0)
+	  usage();
+	num = ULONG_MAX;
+	break;
+      case 2:
+        num = atol(*argv);
+	break;
+      default:
+	usage();
+      } /* switch */
     }
   }
-  switch (argc) {
-  case 0:  /* no numeric args */
-    per=1; 
-    num=0;
-    break;
-  case 1:
-    per=(unsigned) args[0];
-    num=ULONG_MAX;
-    break;
-  case 2:
-    per=(unsigned) args[0];
-    num=args[1];
-    break;
-  default : 
-    usage();
-    break;
-  }
 
   if (moreheaders) {
       int tmp=winhi()-3;
