@@ -6,7 +6,7 @@
  * Slighty modified, and corrected, Francois Pinard, 91-04.
  */
 
-#include <curses.h>
+#include <ncurses.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -28,7 +28,7 @@ extern char *ctime ();
 
 char *program_name;
 
-void
+int
 main (argc, argv)
      int argc;
 char *argv[];
@@ -129,7 +129,7 @@ char *argv[];
   echo ();
   nl ();
   endwin ();
-  exit (0);
+  return (0);
 }
 
 /*------------------------------------.
