@@ -141,9 +141,9 @@ XtInputId id;
  * Main routine. Opens windows and files, sets up callback routines
  *
  **************************************************************************/
-void main(int argc, char **argv)
+int main(int argc, char **argv)
 {
-  int nargs,i;
+  int nargs;
   Arg arg[10];
   time_t tp;
   struct tm *curtime;
@@ -222,4 +222,6 @@ void main(int argc, char **argv)
   XtAppAddTimeOut(app_con,60000,print_date,NULL);
 
   XtAppMainLoop(app_con);
+  
+  return (0); /* never reached */
 }
